module BranchableCDNAssets
  module Middleman

    class Extension < ::Middleman::Extension
      self.supports_multiple_instances = true

      option :id,                     nil,  'name of this cdn'
      option :environments,            {},  'environment specific configuration'
      option :cloudfront,             nil,  'access keys for cloudfront account'
      option :cdn_dir,              'cdn',  'source directory for cdn files'
      option :production_branch, 'master',  'which branch is the production branch'
      option :default_env,      'staging',  'the default environment if none is set'
      option :file_filter,            nil,  'filter for managed files'
      option :allow_local,          false,  'allow local files to remain on deploy checks'

      def after_configuration
        if defined?(::Sass::Script::Functions)
          ::Sass::Script::Functions.send :include, SassHelpers
        end
        if defined?(::SassC::Script::Functions)
          ::SassC::Script::Functions.send :include, SassHelpers
        end
      end

      def manipulate_resource_list resources
        resources + local_files_as_resources
      end

      def config
        data = {
          production_branch: options.production_branch,
          default_env:       options.default_env,
          environments:      options.environments,
          dir:               options.cdn_dir,
          file_filter:       options.file_filter,
          allow_local:       options.allow_local
        }
        if options.cloudfront
          data.merge! invalidators: {
            cloudfront: options.cloudfront
          }
        end
        Config.new data
      end

      def file_manager
        BranchableCDNAssets::FileManager.new config
      end

      def id
        options.id || self.object_id.to_s
      end

      helpers do
        include Helpers
      end

      private

        def local_files_as_resources
          return [] if app.build?

          file_manager.list( :local ).map do |file|
            ::Middleman::Sitemap::Resource.new(
              app.sitemap,
              File.join( "assets/cdn", id, file ),
              File.join( app.root, options.cdn_dir, file )
            )
          end
        end

    end
    ::Middleman::Extensions.register(:cdn_assets, BranchableCDNAssets::Middleman::Extension)

  end
end